"""CMS apphook for the ``enquiry`` app."""
from django.utils.translation import ugettext_lazy as _

from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool


class EnquiryApphook(CMSApp):
    name = _("Enquiry Apphook")
    urls = ["enquiry.urls"]


apphook_pool.register(EnquiryApphook)



"""Admin classes for the ``enquiry`` app."""
from django.contrib import admin

from simple_translation.admin import TranslationAdmin

from enquiry.models import Answer, Enquiry, Vote


class AnswerAdmin(TranslationAdmin):
    pass


class EnquiryAdmin(TranslationAdmin):
    pass


admin.site.register(Answer, AnswerAdmin)
admin.site.register(Enquiry, EnquiryAdmin)
admin.site.register(Vote)



# flake8: noqa
# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Enquiry'
        db.create_table('enquiry_enquiry', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('creation_date', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name='enquiries', to=orm['auth.User'])),
            ('start_date', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime(2013, 2, 6, 0, 0))),
            ('end_date', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime(2013, 2, 13, 0, 0))),
            ('allow_anonymous', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('enquiry', ['Enquiry'])

        # Adding model 'EnquiryTrans'
        db.create_table('enquiry_enquirytrans', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('question', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('enquiry', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['enquiry.Enquiry'])),
            ('language', self.gf('django.db.models.fields.CharField')(max_length=2)),
        ))
        db.send_create_signal('enquiry', ['EnquiryTrans'])

        # Adding model 'Answer'
        db.create_table('enquiry_answer', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('enquiry', self.gf('django.db.models.fields.related.ForeignKey')(related_name='answers', to=orm['enquiry.Enquiry'])),
        ))
        db.send_create_signal('enquiry', ['Answer'])

        # Adding model 'AnswerTrans'
        db.create_table('enquiry_answertrans', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('text', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('answer', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['enquiry.Answer'])),
            ('language', self.gf('django.db.models.fields.CharField')(max_length=2)),
        ))
        db.send_create_signal('enquiry', ['AnswerTrans'])

        # Adding model 'Vote'
        db.create_table('enquiry_vote', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('answer', self.gf('django.db.models.fields.related.ForeignKey')(related_name='votes', to=orm['enquiry.Answer'])),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(related_name='votes', to=orm['auth.User'])),
        ))
        db.send_create_signal('enquiry', ['Vote'])


    def backwards(self, orm):
        # Deleting model 'Enquiry'
        db.delete_table('enquiry_enquiry')

        # Deleting model 'EnquiryTrans'
        db.delete_table('enquiry_enquirytrans')

        # Deleting model 'Answer'
        db.delete_table('enquiry_answer')

        # Deleting model 'AnswerTrans'
        db.delete_table('enquiry_answertrans')

        # Deleting model 'Vote'
        db.delete_table('enquiry_vote')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'enquiry.answer': {
            'Meta': {'object_name': 'Answer'},
            'enquiry': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'answers'", 'to': "orm['enquiry.Enquiry']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'enquiry.answertrans': {
            'Meta': {'object_name': 'AnswerTrans'},
            'answer': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['enquiry.Answer']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'text': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'enquiry.enquiry': {
            'Meta': {'object_name': 'Enquiry'},
            'allow_anonymous': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'enquiries'", 'to': "orm['auth.User']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'end_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2013, 2, 13, 0, 0)'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'start_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2013, 2, 6, 0, 0)'})
        },
        'enquiry.enquirytrans': {
            'Meta': {'object_name': 'EnquiryTrans'},
            'enquiry': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['enquiry.Enquiry']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'question': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'enquiry.vote': {
            'Meta': {'object_name': 'Vote'},
            'answer': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'votes'", 'to': "orm['enquiry.Answer']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'votes'", 'to': "orm['auth.User']"})
        }
    }

    complete_apps = ['enquiry']


"""Tests for views of the ``enquiry``` application."""
from django.test import TestCase

from django_libs.tests.factories import UserFactory
from django_libs.tests.mixins import ViewTestMixin

from enquiry.models import Vote
from enquiry.tests.factories import (
    AnswerTransENFactory,
    EnquiryTransENFactory,
)


class EnquirySubmitViewTestCase(ViewTestMixin, TestCase):
    """Tests for the ``EnquirySubmitView`` view."""
    longMessage = True

    def setUp(self):
        self.user = UserFactory()
        self.answer_translated = AnswerTransENFactory()
        self.enquiry_translated = EnquiryTransENFactory(
            enquiry=self.answer_translated.answer.enquiry)
        self.enquiry_translated_2 = EnquiryTransENFactory()

    def get_view_name(self):
        return 'enquiry_detail'

    def get_view_kwargs(self):
        return {'pk': self.enquiry_translated.enquiry.pk}

    def test_view(self):
        self.is_callable()

        # 404: Not logged in (enquiry disallows anonymous votes)
        data = {'answer_pk': 'abc'}
        self.is_not_callable(method='post', data=data)

        # 404: Posted answer id not int()
        self.login(self.user)
        self.is_not_callable(method='post', data=data)

        # 404: Answer not found
        data.update({'answer_pk': 999})
        self.is_not_callable(method='post', data=data)

        # 404: Answer doesn't match the enquiry
        data.update({'answer_pk': self.answer_translated.pk})
        self.is_not_callable(method='post', data=data, kwargs={
            'pk': self.enquiry_translated_2.pk})

        # 200: Answer matches enquiry
        self.is_callable(method='post', data=data)
        self.assertEqual(Vote.objects.all().count(), 1)
        self.assertEqual(Vote.objects.all()[0].user, self.user)
        self.assertEqual(Vote.objects.all()[0].session_key,
                         self.client.session.session_key)

        # 200: User can post, but no new vote should be created
        self.is_callable(method='post', data=data)
        self.assertEqual(Vote.objects.all().count(), 1)

        # 200: User can still view the enquiry
        self.is_callable()

        # 404: Should be callable as anonymous
        self.is_not_callable(method='post', data=data, anonymous=True)

        # 200: Should be callable as anonymous
        self.is_callable(data=data, anonymous=True)
